
import uuid

from django.conf import settings
from django.db import models
from django.contrib.auth.models import User, Group
from django.utils.translation import gettext_lazy as _

from uhepp_api.models import Token

PRIVATE_LEVEL = 10
INTERNAL_LEVEL = 20
PUBLIC_LEVEL = 30
VISIBILITY_LEVELS = [
    (PRIVATE_LEVEL, _("Accessible only by you")),
    (INTERNAL_LEVEL, _("Accessible only by members of your collaboration")),
    (PUBLIC_LEVEL, _("Publicly accessible")),
]

class Collection(models.Model):
    title = models.CharField(_("Collection title"), max_length=100)
    slug = models.SlugField(_("Collection URL slug"))
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='collections'
    )
    description = models.TextField(
        _("Collection description"),
        null=True,
        blank=True
    )
    visibility = models.IntegerField(
        _("Collection visibility level"),
        choices=VISIBILITY_LEVELS,
        default=PRIVATE_LEVEL,
    )

    def __str__(self):
        return f"{self.owner}: {self.title}"

class Plot(models.Model):
    """Database record of a uhepp compliant plot"""
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4)
    uhepp = models.JSONField()
    collection = models.ForeignKey(
        Collection,
        on_delete=models.CASCADE,
        related_name='plots'
    )

    def __str__(self):
        try:
            metadata = self.uhepp.get("metadata", {})
            filename = metadata.get("filename")
            return filename or str(self.uuid)
        except (AttributeError, KeyError):
            return str(self.uuid)

class Profile(models.Model):
    """User profile"""
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        primary_key=True
    )
    cern_upn = models.CharField(
        _("User Principal Name"),
        max_length=100,
        unique=True
    )
    home_institute = models.CharField(
        _("Home Institute"),
        max_length=100,
        null=True,
        blank=True
    )
    cern_person_id = models.IntegerField(
        _("CERN Person ID"),
        null=True,
        blank=True
    )
    visibility = models.IntegerField(
        _("Profile visibility level"),
        choices=VISIBILITY_LEVELS,
        default=PRIVATE_LEVEL,
    )

    def __str__(self):
        return self.cern_upn
